###
### Date: 23 October, 1996
###       29 January, 1997 (added in subscripting capability)
###
###
### sample code is roc(control~cass+compass+compas2,data=d1)
###


roc <- function(formula=formula(data), data=sys.parent(), subset, 
                predict.pos=1, x=T, y=T)

    {

    #  Like the function glm (and others) store the current call and reset
    #  all options that match.call does not know about to NULL. 

    call <- match.call()
    m <- match.call(expand.dots = F)
    m$predict.pos <- m$se.method <- m$x <- m$y <- m$... <- NULL
    Terms <- terms(formula, data=data)
    m$formula <- Terms
    ord <- attr(Terms, 'order')
    m[[1]] <- as.name('model.frame')
    m <- eval(m, sys.parent())

    ## main variables needed here
    status <- model.extract(m, 'response')
    nmark <- ncol(m)-1
    X <- matrix(model.matrix(Terms,m)[,-1],ncol=nmark)
  
    ## if there are missings in status, fix up X and status
    if(sum(is.na(status))>0) {
       X <- X[!is.na(status),]
       status <- status[!is.na(status)]
       }
  
    ### check the variables/formula and see if they meet our criteria
    if(length(ord) & any(ord !=1))
		stop("Interaction terms are not valid for this function")
    if(!is.numeric(X))
              stop("All markers must be numeric")
    if(min(status)<0 | max(status)>1)
              stop("Status endpoint must have the values 0 and 1")
    if(!is.numeric(status))
              stop("Status endpoint must be numeric")
    if(predict.pos<0 | predict.pos>4)
              stop("predict.pos must be 1, 2, 3, or 4")
  

    nobs <- nrow(X)
    num1 <- sum(status)
    num0 <- length(status) - sum(status)

    sensitivity <- specificity <- matrix(NA,nrow=nobs,ncol=nmark)
    t.n <- t.p <- f.n <- f.p <- matrix(NA,nrow=nobs,ncol=nmark)

    for(k in 1:nmark){

       marker <- X[,k]
       order.mark <- order(marker)

       temp.mark <- marker[order.mark]
       temp.status <- status[order.mark]
       temp.uniq <- unique(temp.mark)

       spec <- sens <- cutpoint <- matrix(NA,nrow=length(temp.uniq))
       a.tp <- c.fn <- d.tn <- b.fp <- matrix(NA,nrow=length(temp.uniq))

       for(j in 1:length(temp.uniq)) {
          cutoff <- temp.uniq[j]

	  if(predict.pos==1) {
	     sens.vec <- temp.status[temp.mark>=cutoff]
	     spec.vec <- temp.status[temp.mark<cutoff]
	     }
          if(predict.pos==2) {
  	     sens.vec <- temp.status[temp.mark>cutoff]
	     spec.vec <- temp.status[temp.mark<=cutoff]
             }
          if(predict.pos==3) {
  	     sens.vec <- temp.status[temp.mark<=cutoff]
	     spec.vec <- temp.status[temp.mark>cutoff]
             }
          if(predict.pos==4) {
  	     sens.vec <- temp.status[temp.mark<cutoff]
	     spec.vec <- temp.status[temp.mark>=cutoff]
             }
	
          spec[j] <- (length(spec.vec)-sum(spec.vec))/num0
          sens[j] <- sum(sens.vec)/num1
  
          cutpoint[j] <- cutoff
          a.tp[j] <- num1*sens[j]
	  c.fn[j] <- num1 - a.tp[j]
	  d.tn[j] <- num0*spec[j]
	  b.fp[j] <- num0 - d.tn[j]
	}
       sensitivity[,k] <- sens[match(marker,temp.uniq)]
       specificity[,k] <- spec[match(marker,temp.uniq)]
       t.p[,k] <- a.tp[match(marker,temp.uniq)]
       t.n[,k] <- d.tn[match(marker,temp.uniq)]
       f.p[,k] <- b.fp[match(marker,temp.uniq)]
       f.n[,k] <- c.fn[match(marker,temp.uniq)]

     }

   ## results that are saved

   outdata <- list()
   outdata$method <- 'roc'
   outdata$call <- call
   outdata$formula <- formula
   outdata$terms <- Terms
   outdata$predict.pos <- predict.pos
   if(y==T) outdata$y <- status
   if(x==T) outdata$x <- X
   outdata$sensitivity <- sensitivity
   outdata$specificity <- specificity
   outdata$true.p <- t.p
   outdata$true.n <- t.n
   outdata$false.p <- f.p
   outdata$false.n <- f.n
   class(outdata) <- 'roc'
   return(outdata)

  }

## need quotes around function name because bracket has other meanings
## most users don't need to know about this function

"[.roc" <- function(x,i, ..., drop=F) {
    z<- list(method=x$method, call=x$call, formula=x$formula, 
	     terms=x$terms[i],
	     predict.pos=x$predict.pos, y=x$y, x=x$x[,i,drop=drop], 
	     sensitivity=x$sensitivity[,i,drop=drop], 
	     specificity=x$specificity[,i,drop=drop],
	     true.p=x$true.p[,i,drop=drop], true.n=x$true.n[,i,drop=drop], 
	     false.p=x$false.p[,i,drop=drop],false.n=x$false.n[,i,drop=drop])

    attributes(z) <- attributes(x)
    z
    }

